/*! \file ks0073.h
	\brief header for AVR-GCC library for interfacing a KS0073 based LCD via hardware SPI
	
	No error checking/handling implemented at all
	Author: J. Gerlach - gerlach_joerg(at)web.de
*/

#ifndef KS0073_H_
#define KS0073_H_ 1
//#define DEBUG 1

#include <stdint.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <util/delay.h>

/*
	These macros need to be configured!
*/
#define SPI_DDR DDRB
#define SPI_PORT PORTB
/*	
for safety reasons only: AVR enters slave-mode if SS pin becomes input-low */
#define SS PB4
#define MOSI PB5
#define MISO PB6
#define SCK PB7
// the CS-signal to the LCD, low-active
#define CS_DDR DDRA
#define CS_PORT PORTA
#define CS PA2

#define DISPLAYROWS 4
#define DISPLAYCOLS 20
//! Clear CS - activate LCD
#define CLR_CS() CS_PORT &= ~(1<<CS)
//! set CS - deactivate LCD
#define SET_CS() CS_PORT |= (1<<CS)

// R\W
#define LCD_RW 5
// RS(register select): 0 = Cmd, 1 = Data
#define LCD_RS 6
/*! the startby contains 1s in the 5 lower bits

	resets internal communication of the ks0073
*/
#define STARTBYTE 0x1F
#define DUMMYBYTE 0x00
#define LCD_BF 0x80

//! delay-loop for slower instructions CLEARDISP and RETHOME
#define LCD_WAIT_LONG _delay_ms(2)

//! delay-loop for normal LCD instructions
#define LCD_WAIT_SHORT _delay_us(50)

#define STARTTIME 20
/*! \def STARTUPDELAY 
	\brief 20ms delay at startup required
	max. delay is limited to 262.14[ms]/F_CPU[MHz] (cp. util/delay.h)
	so this delay is added if higher F_CPU values are used.
*/

#if((262/(F_CPU/1000000UL)) < STARTTIME)
#	define AUXWAIT STARTTIME-(262/(F_CPU/1000000UL))
#	define STARTUPDELAY() _delay_ms(AUXWAIT); _delay_ms(20)
#else
#	define STARTUPDELAY() _delay_ms(20)
#endif

/*! Macro to send stringliteral from flash

	Just simplifying macro
*/
#define lcd_puts_P(__s) lcd_puts_p(PSTR(__s))

//! Wait for busy-flag being cleared
#define LCD_BUSYWAIT() while(lcd_get_status() & LCD_BF)


/*! Init-sequence, taken from dip204-6 datasheet from Electronic Assembly

	Set 8-bit mode, 4 rows, clear all icons and display. 
	Takes a while due to the 20ms startup delay
*/
void lcd_init(void);

//!	Send a single command to display controller
void lcd_put_cmd(uint8_t cmd);

//! Send a single data byte to display controller
void lcd_put_data(uint8_t data);

/*!	Writes a 0-terminated string to LCD, 

	special characters are ignored (yet)
*/
void lcd_puts(char *lcds);

/*!	Writes a 0-terminated constant string from flash to LCD, 

	to send literals use lcd_puts_P() instead; special characters are ignored (yet)
*/
void lcd_puts_p(const char *progmem_s );

/*!	set cursor (x,y) position on display
	
	set adress in DDRAM, position is 0-based (of course).
	the lcd_puts..() functions can then be used to write text to display
*/
void lcd_gotoxy(uint8_t col, uint8_t row);

/*!	read instruction register from LCD

	read LCD busy state from MSB, lower bits are current cursor address
*/
uint8_t lcd_get_status(void);



//	Macros for KS0073 instructions

/*	Somewhere between readable, the datasheet naming conventions and sense
	ENH_.. are usable only after FUNC_SET|LCD_RE being sent
*/
#define CLEARDISP 0x01
#define RETHOME 0x02
#define POWERDOWN 0x02
#define ENTRYMODE 0x04
#define ENH_ENTRY 0x06
#define DISP_ON 0x08
#define ENH_EXT_FUNCT 0x08
#define CD_SHIFT 0x10
#define ENH_SHIFTEN 0x10
#define ENH_SCREN 0x10
#define FUNCSET 0x20
#define ENH_FUNCSET 0x20
#define SET_CADDR 0x40
#define ENH_SET_SEGA 0x40
#define SET_DDADDR 0x80
#define ENH_SET_SCR 0x80

#define LCD_PD 0x01

#define LCD_INC 0x02
#define LCD_SHIFT 0x01

#define LCD_BID 0x01

#define LCD_DISP 0x04
#define LCD_CUR 0x02
#define LCD_BLINK 0x01

#define LCD_FONTW 0x04	// FW
#define LCD_6DOT 0x04
#define LCD_BW 0x02
#define LCD_CURINV 0x02
#define LCD_NW 0x01

#define LCD_SC_SHIFT 0x08
#define LCD_RSH 0x04

#define LCD_DS1 0x01
#define LCD_DS2 0x02
#define LCD_DS3 0x04
#define LCD_DS4 0x08
#define LCD_HS1 0x01
#define LCD_HS2 0x02
#define LCD_HS3 0x04
#define LCD_HS4 0x08
#define LCD_DL 0x10
#define LCD_N 0x08
#define LCD_RE 0x04
#define LCD_DH 0x02
#define LCD_REV 0x01
#define LCD_BE 0x02
#define LCD_LP 0x01

#define DD_MSK 0x7F
#define CG_MSK 0x3F
#define SEGMSK 0x0F
#define SCQUMSK 0x3F
#endif
